% Input to this program:
% edgeu = edges in GU
% confu = bidirected edges in GU
% u = known u, if there is one 
% node(1..n)


% edge1 =  edge in the learned graph G1
% Guess edge1:s
{ edge1(X,Y) } :- node(X),node(Y).

% For now there are no confs

% Derive all edges up to length U of G1
edge(X,Y,1) :- edge1(X,Y).
edge(X,Y,L) :- edge(X,Z,L-1),edge1(Z,Y),
	       L <= U, u(U).

% Edges of length U, 
% are edgeu:s of the learning result 
derived_edgeu(X,Y) :- edge(X,Y,L), u(L).

% Find the confus that would show up for the G1
% Only considered when X < Y here 
derived_confu(X,Y) :- edge(Z,X,L), edge(Z,Y,L),
                       node(X),node(Y),node(Z), 
                            X < Y, L < U, u(U).

% Find inconsistencies with input

failed_edgeu(X,Y,W) :- edgeu(X,Y,W), 
                       not derived_edgeu(X,Y),
                       node(X),node(Y).

failed_edgeu(X,Y,W) :- no_edgeu(X,Y,W),
                       derived_edgeu(X,Y),
                       node(X),node(Y).

failed_confu(X,Y,W):- confu(X,Y,W),
                      not derived_confu(X,Y),  
                      node(X),node(Y), X < Y.

failed_confu(X,Y,W) :- no_confu(X,Y,W),
                       derived_confu(X,Y),  
                       node(X),node(Y), X < Y.
      
% objective function:
% suffer the following losses

:~ failed_edgeu(X,Y,W). [W,X,Y,1]
:~ failed_confu(X,Y,W). [W,X,Y,2]
       
#show.
#show failed_confu/3.
#show failed_edgeu/3.
#show edge1/2.
#show u/1.
